# Proposer - Create Transaction Payload

The proposer is a fiduciary whose responsibility is to make sound financial decisions on behalf of a business and determine where funds are moving from, where to and in which amount. The reasons for sending funds may range across settlement, exchanging, staking and more.

The proposer MUST clearly define, at a minimum:

* Token Name (SOL, PYTH-SPL, ETH, ETH-PYTH, BTC etc.)

* FROM address

* TO address

* Amount

* Date + Time

The proposer must combine these values into a single message, which can be a simple JSON file, and sign it using a well known PGP key.

## Requirements

* If necessary, provision a PGP key pair to a smart card using the guide in the [Appendix: Generating PGP Keypair & Provisioning Smart Card](#generating-pgp-keypair--provisioning-smart-card)

* Ensure that the computer is configured to sign commits with the desired key. Refer to the [Appendix: Git Commit Signing Configuration](#git-commit-signing-configuration)

* Clone the [Ceremonies Repository](../provisioner/provision-ceremonies-repository.md) for your organization to the machine

## Procedure

1. Define a new file "<date:time>-<currency>.tx.json", for example "16:40-PYTH-SPL.tx.json"

1. Create a new directory in the `ceremonies` repository for the date on which the ceremony for the transaction will take place if it doesn't already exist, for example `2024-01-01/`

1. Collect data for the transaction being sent, and structure it according to the template below, replacing values with valid ones. The values have to come from a organization approved list of values, for each field, except for `datetime` which is just the current date and time.
    ```json
    {
        "token-name": "<name>",
        "token-amount": "<amount>",
        "from-address": "<address>",
        "to-address": "<address>",
        "datetime": "<date:time>"
    }
    ```

    Example data object:
    ```json
    {
        "token-name": "PYTH-SLN",
        "token-amount": "42",
        "from_address": "2Z72E62atYfpatQeqPvHZMaabmuz664xq5MRWv9xM5NX",
        "to_address": "BNQr6T2UAuEPux1fuiygM6chrT5GkHKaMWeTTaRLmR7g",
        "datetime": "<date:time>"
    }
    ```

1. Sign the data in the CLI using `gpg` or another OpenPGP implementation:

    * `gpg --clearsign <file>`

1. Notify relevant individuals that there are new transactions queued up, and that a ceremony should be scheduled. This can be automated in the future so that when a commit is made or PR opened, others are notified, for example using a incident management tool(TODO).

## Appendix

### Git Commit Signing Configuration

{{ #include ../../../../component-documents/git-commit-signing.md:steps }}

### Generating PGP Keypair & Provisioning Smart Card

{{ #include ../../../../component-documents/openpgp-setup.md:steps-keyfork }}
